@@ -15,7 +15,7 @@
  */
 
 /**
- * $Id: 4b83f56ea1a3fb48163e2ceeecac35716a578855 $
+ * $Id: 6228438c47f9f5941e8a0535f44dc152d1b04414 $
  * @file main/client.c
  * @brief Manage clients allowed to communicate with the server.
  *
@@ -24,7 +24,7 @@
  * @copyright 2000 Alan DeKok <aland@ox.org>
  * @copyright 2000 Miquel van Smoorenburg <miquels@cistron.nl>
  */
-RCSID("$Id: 4b83f56ea1a3fb48163e2ceeecac35716a578855 $")
+RCSID("$Id: 6228438c47f9f5941e8a0535f44dc152d1b04414 $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/rad_assert.h>
@@ -55,10 +55,6 @@ static int		tree_num_max = 0;
 #endif
 static RADCLIENT_LIST	*root_clients = NULL;
 
-#ifdef WITH_DYNAMIC_CLIENTS
-static fr_fifo_t	*deleted_clients = NULL;
-#endif
-
 /*
  *	Callback for freeing a client.
  */
@@ -66,38 +62,6 @@ void client_free(RADCLIENT *client)
 {
 	if (!client) return;
 
-#ifdef WITH_DYNAMIC_CLIENTS
-	if (client->dynamic == 2) {
-		time_t now;
-
-		if (!deleted_clients) {
-			deleted_clients = fr_fifo_create(NULL, 1024, (void (*)(void *))client_free);
-			if (!deleted_clients) return; /* MEMLEAK */
-		}
-
-		/*
-		 *	Mark it as in the fifo, and remember when we
-		 *	pushed it.
-		 */
-		client->dynamic = 3;
-		client->created = now = time(NULL); /* re-set it */
-		fr_fifo_push(deleted_clients, client);
-
-		/*
-		 *	Peek at the head of the fifo.  If it might
-		 *	still be in use, return.  Otherwise, pop it
-		 *	from the queue and delete it.
-		 */
-		client = fr_fifo_peek(deleted_clients);
-		rad_assert(client != NULL);
-
-		if ((client->created + 120) >= now) return;
-
-		client = fr_fifo_pop(deleted_clients);
-		rad_assert(client != NULL);
-	}
-#endif
-
 	talloc_free(client);
 }
 
@@ -246,7 +210,11 @@ bool client_add(RADCLIENT_LIST *clients, RADCLIENT *client)
 			 *	to the global client list.
 			 */
 			subcs = cf_section_sub_find(cs, "listen");
-			if (!subcs) goto global_clients;
+			if (!subcs) {
+				DEBUG("No 'listen' section in virtual server %s.  Adding client to global client list",
+				      client->server);
+				goto global_clients;
+			}
 
 			/*
 			 *	If the client list already exists, use that.
@@ -386,8 +354,6 @@ void client_delete(RADCLIENT_LIST *clients, RADCLIENT *client)
 
 	rad_assert(client->ipaddr.prefix <= 128);
 
-	client->dynamic = 2;	/* signal to client_free */
-
 #ifdef WITH_STATS
 	rbtree_deletebydata(tree_num, client);
 #endif
@@ -1217,6 +1183,14 @@ RADCLIENT *client_afrom_request(RADCLIENT_LIST *clients, REQUEST *request)
 
 	if (!clients || !request) return NULL;
 
+	/*
+	 *	Hack for the "dynamic_clients" module.
+	 */
+	if (request->client->dynamic) {
+		c = request->client;
+		goto validate;
+	}
+
 	snprintf(buffer, sizeof(buffer), "dynamic%i", cnt++);
 
 	c = talloc_zero(clients, RADCLIENT);
@@ -1428,6 +1402,7 @@ RADCLIENT *client_afrom_request(RADCLIENT_LIST *clients, REQUEST *request)
 	}
 	REXDENT();
 
+validate:
 	if (c->ipaddr.af == AF_UNSPEC) {
 		RERROR("Cannot add client %s: No IP address was specified.",
 		       ip_ntoh(&request->packet->src_ipaddr, buffer, sizeof(buffer)));
