@@ -15,7 +15,7 @@
  */
 
 /**
- * $Id: b5bbe002b050dd56401cca03934b52e89fb7c970 $
+ * $Id: 998751f9db377a0f6397b8c56e6abef4af69a620 $
  *
  * @file xlat.c
  * @brief String expansion ("translation"). Implements %Attribute -> value
@@ -24,7 +24,7 @@
  * @copyright 2000  Alan DeKok <aland@ox.org>
  */
 
-RCSID("$Id: b5bbe002b050dd56401cca03934b52e89fb7c970 $")
+RCSID("$Id: 998751f9db377a0f6397b8c56e6abef4af69a620 $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/parser.h>
@@ -486,6 +486,7 @@ static ssize_t xlat_debug_attr(UNUSED void *instance, REQUEST *request, char con
 			type++;
 		}
 	}
+	REXDENT();
 
 	*out = '\0';
 	return 0;
@@ -1468,7 +1469,7 @@ static ssize_t xlat_tokenize_literal(TALLOC_CTX *ctx, char *fmt, xlat_exp_t **he
 			ssize_t slen;
 			xlat_exp_t *next;
 
-			if (!p[1] || !strchr("%}dlmntDGHISTYv", p[1])) {
+			if (!p[1] || !strchr("%}cdelmntCDGHIMSTYv", p[1])) {
 				talloc_free(node);
 				*error = "Invalid variable expansion";
 				p++;
@@ -1913,6 +1914,12 @@ static char *xlat_getvp(TALLOC_CTX *ctx, REQUEST *request, vp_tmpl_t const *vpt,
 				code = request->reply->code;
 			}
 
+		if (!code) return NULL;
+
+		if (code >= FR_MAX_PACKET_CODE) {
+			return talloc_typed_asprintf(ctx, "%d", packet->code);
+		}
+
 		return talloc_typed_strdup(ctx, fr_packet_codes[code]);
 	}
 
@@ -2126,11 +2133,21 @@ static char *xlat_aprint(TALLOC_CTX *ctx, REQUEST *request, xlat_exp_t const * c
 			str[1] = '\0';
 			break;
 
+		case 'c':	/* current epoch time seconds */
+			snprintf(str, freespace, "%" PRIu64, (uint64_t) time(NULL));
+			break;
+
 		case 'd': /* request day */
 			if (!localtime_r(&when, &ts)) goto error;
 			strftime(str, freespace, "%d", &ts);
 			break;
 
+		case 'e': /* request second */
+			if (!localtime_r(&when, &ts)) goto error;
+
+			snprintf(str, freespace, "%d", ts.tm_sec);
+			break;
+
 		case 'l': /* request timestamp */
 			snprintf(str, freespace, "%lu",
 				 (unsigned long) when);
@@ -2151,6 +2168,16 @@ static char *xlat_aprint(TALLOC_CTX *ctx, REQUEST *request, xlat_exp_t const * c
 			if (nl) *nl = '\0';
 			break;
 
+		case 'C':	/* current epoch time microseconds */
+			{
+				struct timeval tv;
+
+				gettimeofday(&tv, NULL);
+
+				snprintf(str, freespace, "%" PRIu64, (uint64_t) tv.tv_usec);
+			}
+			break;
+
 		case 'D': /* request date */
 			if (!localtime_r(&when, &ts)) goto error;
 			strftime(str, freespace, "%Y%m%d", &ts);
@@ -2172,6 +2199,10 @@ static char *xlat_aprint(TALLOC_CTX *ctx, REQUEST *request, xlat_exp_t const * c
 			}
 			break;
 
+		case 'M': /* request microsecond component */
+			snprintf(str, freespace, "%06u", (unsigned int) usec);
+			break;
+
 		case 'S': /* request timestamp in SQL format*/
 			if (!localtime_r(&when, &ts)) goto error;
 			strftime(str, freespace, "%Y-%m-%d %H:%M:%S", &ts);
@@ -2229,6 +2260,18 @@ static char *xlat_aprint(TALLOC_CTX *ctx, REQUEST *request, xlat_exp_t const * c
 		}
 		RDEBUG2("EXPAND %s", node->xlat->name);
 		RDEBUG2("   --> %s", str);
+
+		/*
+		 *	Resize the buffer to the correct size.
+		 */
+		if (rcode == 0) {
+			talloc_free(str);
+			str = talloc_strdup(ctx, "");
+		} else if (rcode < 2047) {
+			child = talloc_memdup(ctx, str, rcode + 1);
+			talloc_free(str);
+			str = child;
+		}
 		break;
 
 	case XLAT_MODULE:
