@@ -1,7 +1,7 @@
 /*
  * eap_fast.c  contains the interfaces that are called from the main handler
  *
- * Version:     $Id: 44e6bce7ad10c8153802ab92890c35f5448b0ec5 $
+ * Version:     $Id: b0953aa1d4e15cc9d89fd13c7019d774f38ce59d $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  *   Copyright 2016 The FreeRADIUS server project
  */
 
-RCSID("$Id: 44e6bce7ad10c8153802ab92890c35f5448b0ec5 $")
+RCSID("$Id: b0953aa1d4e15cc9d89fd13c7019d774f38ce59d $")
 
 #include "eap_fast.h"
 #include "eap_fast_crypto.h"
@@ -44,7 +44,7 @@ static int openssl_get_keyblock_size(REQUEST *request, SSL *ssl)
 {
 	const EVP_CIPHER *c;
 	const EVP_MD *h;
-#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
+#if OPENSSL_VERSION_NUMBER < 0x10100000L && !defined(LIBRESSL_VERSION_NUMBER)
 	int md_size;
 
 	if (ssl->enc_read_ctx == NULL || ssl->enc_read_ctx->cipher == NULL ||
@@ -773,10 +773,21 @@ static rlm_rcode_t CC_HINT(nonnull) process_reply( eap_handler_t *eap_session,
 			/* FIXME must be a better way to capture/re-derive this later for ISK */
 			switch (vp->da->attr) {
 			case PW_MSCHAP_MPPE_SEND_KEY:
+				if (vp->vp_length != CHAP_VALUE_LENGTH) {
+				wrong_length:
+					REDEBUG("Found %s with incorrect length.  Expected %u, got %zu",
+						vp->da->name, 16, vp->vp_length);
+					rcode = RLM_MODULE_INVALID;
+					break;
+				}
+
+
 				memcpy(t->isk.mppe_send, vp->vp_octets, CHAP_VALUE_LENGTH);
 				break;
 
 			case PW_MSCHAP_MPPE_RECV_KEY:
+				if (vp->length != CHAP_VALUE_LENGTH) goto wrong_length;
+
 				memcpy(t->isk.mppe_recv, vp->vp_octets, CHAP_VALUE_LENGTH);
 				break;
 
