@@ -15,7 +15,7 @@
  */
 
 /**
- * $Id: c6fbedf1ace4347a6ca0a0deef8cc0548e897bae $
+ * $Id: fdd6f01ecf008933de231c22787cc0947ad9fbc8 $
  * @file rlm_ldap.c
  * @brief LDAP authorization and authentication module.
  *
@@ -27,7 +27,7 @@
  * @copyright 2012 Alan DeKok <aland@freeradius.org>
  * @copyright 1999-2013 The FreeRADIUS Server Project.
  */
-RCSID("$Id: c6fbedf1ace4347a6ca0a0deef8cc0548e897bae $")
+RCSID("$Id: fdd6f01ecf008933de231c22787cc0947ad9fbc8 $")
 
 #include	<freeradius-devel/rad_assert.h>
 
@@ -153,6 +153,7 @@ static CONF_PARSER group_config[] = {
 	{ "cacheable_name", FR_CONF_OFFSET(PW_TYPE_BOOLEAN, rlm_ldap_t, cacheable_group_name), "no" },
 	{ "cacheable_dn", FR_CONF_OFFSET(PW_TYPE_BOOLEAN, rlm_ldap_t, cacheable_group_dn), "no" },
 	{ "cache_attribute", FR_CONF_OFFSET(PW_TYPE_STRING, rlm_ldap_t, cache_attribute), NULL },
+	{ "allow_dangling_group_ref", FR_CONF_OFFSET(PW_TYPE_BOOLEAN, rlm_ldap_t, allow_dangling_group_refs), "no" },
 	CONF_PARSER_TERMINATOR
 };
 
@@ -225,6 +226,8 @@ static const CONF_PARSER module_config[] = {
 
 	{ "valuepair_attribute", FR_CONF_OFFSET(PW_TYPE_STRING, rlm_ldap_t, valuepair_attr), NULL },
 
+	{ "user_dn", FR_CONF_OFFSET(PW_TYPE_STRING, rlm_ldap_t, user_dn), NULL },
+
 #ifdef WITH_EDIR
 	/* support for eDirectory Universal Password */
 	{ "edir", FR_CONF_OFFSET(PW_TYPE_BOOLEAN, rlm_ldap_t, edir), NULL }, /* NULL defaults to "no" */
@@ -615,6 +618,24 @@ static int mod_bootstrap(CONF_SECTION *conf, void *instance)
 		inst->cache_da = inst->group_da;	/* Default to the group_da */
 	}
 
+	if (!inst->user_dn || !*inst->user_dn) {
+		inst->user_dn = talloc_strdup(inst, "LDAP-UserDn");
+	}
+
+	/*
+	 *	Check or create the LDAP-UserDn attribute.
+	 */
+	if (inst->user_dn) {
+		ATTR_FLAGS flags;
+
+		memset(&flags, 0, sizeof(flags));
+		if (dict_addattr(inst->user_dn, -1, 0, PW_TYPE_STRING, flags) < 0) {
+			LDAP_ERR("Error creating %s attribute: %s", inst->user_dn, fr_strerror());
+			return -1;
+		}
+		inst->user_dn_da = dict_attrbyname(inst->user_dn);
+	}
+
 	xlat_register(inst->name, ldap_xlat, rlm_ldap_escape_func, inst);
 	xlat_register("ldapquote", ldapquote_xlat, NULL, inst);
 
