@@ -15,7 +15,7 @@
  */
 
 /**
- * $Id: dff2e86bbda4fa49ef642bb3d13dbde1fdcbdf53 $
+ * $Id: 7f184ad3a43d06815d8ca0fde96c734192b8e50c $
  * @file rlm_pap.c
  * @brief Hashes plaintext passwords to compare against a prehashed reference.
  *
@@ -23,7 +23,7 @@
  * @copyright 2012       Matthew Newton <matthew@newtoncomputing.co.uk>
  * @copyright 2001       Kostas Kalevras <kkalev@noc.ntua.gr>
  */
-RCSID("$Id: dff2e86bbda4fa49ef642bb3d13dbde1fdcbdf53 $")
+RCSID("$Id: 7f184ad3a43d06815d8ca0fde96c734192b8e50c $")
 USES_APPLE_DEPRECATED_API
 
 #include <freeradius-devel/radiusd.h>
@@ -246,10 +246,10 @@ redo:
 		attr = fr_str2int(header_names, buffer, 0);
 		if (!attr) {
 			if (RDEBUG_ENABLED3) {
-				RDEBUG3("Unknown header {%s} in Password-With-Header = \"%s\", re-writing to "
+				RDEBUG3("Unknown header %s in Password-With-Header = \"%s\", re-writing to "
 					"Cleartext-Password", buffer, vp->vp_strvalue);
 			} else {
-				RDEBUG("Unknown header {%s} in Password-With-Header, re-writing to "
+				RDEBUG("Unknown header %s in Password-With-Header, re-writing to "
 				       "Cleartext-Password", buffer);
 			}
 			goto unknown_header;
@@ -357,7 +357,7 @@ static rlm_rcode_t CC_HINT(nonnull) mod_authorize(void *instance, REQUEST *reque
 			 */
 			if (fr_pair_find_by_num(request->config, PW_CLEARTEXT_PASSWORD, 0, TAG_ANY)) {
 				RWDEBUG("Config already contains a \"known good\" password "
-					"(&control:Cleartext-Password).  Ignoring &config:Password-With-Header");
+					"(&control:Cleartext-Password).  Ignoring &control:Password-With-Header");
 				break;
 			}
 
@@ -492,6 +492,11 @@ static rlm_rcode_t CC_HINT(nonnull) mod_authorize(void *instance, REQUEST *reque
 			return RLM_MODULE_NOOP;
 		}
 
+		if (auth_type) {
+			DEBUG("Not doing PAP as Auth-Type is already set.");
+			return RLM_MODULE_NOOP;
+		}
+
 		RWDEBUG("No \"known good\" password found for the user.  Not setting Auth-Type");
 		RWDEBUG("Authentication will fail unless a \"known good\" password is available");
 		return RLM_MODULE_NOOP;
