@@ -15,7 +15,7 @@
  */
 
 /**
- * $Id: 5f694b5519d75e0b82a5bc5c93dae1f022551862 $
+ * $Id: 2a6812bcc6a9c0f4e84264d0d8ea5b726c25604d $
  * @file rlm_python.c
  * @brief Translates requests between the server an a python interpreter.
  *
@@ -25,7 +25,7 @@
  * @copyright 2002  Miguel A.L. Paraz <mparaz@mparaz.com>
  * @copyright 2002  Imperium Technology, Inc.
  */
-RCSID("$Id: 5f694b5519d75e0b82a5bc5c93dae1f022551862 $")
+RCSID("$Id: 2a6812bcc6a9c0f4e84264d0d8ea5b726c25604d $")
 
 #define LOG_PREFIX "rlm_python - "
 
@@ -35,13 +35,14 @@ RCSID("$Id: 5f694b5519d75e0b82a5bc5c93dae1f022551862 $")
 #include <freeradius-devel/rad_assert.h>
 
 #include <Python.h>
+#include <frameobject.h> /* Python header not pulled in by default. */
 #include <dlfcn.h>
 #ifdef HAVE_DL_ITERATE_PHDR
 #include <link.h>
 #endif
 
 #define LIBPYTHON_LINKER_NAME \
-	"libpython" STRINGIFY(PY_MAJOR_VERSION) "." STRINGIFY(PY_MINOR_VERSION) ".so"
+	"libpython" STRINGIFY(PY_MAJOR_VERSION) "." STRINGIFY(PY_MINOR_VERSION) LT_SHREXT
 
 static uint32_t		python_instances = 0;
 static void		*python_dlhandle;
@@ -70,7 +71,6 @@ typedef struct rlm_python_t {
 
 #if PY_VERSION_HEX > 0x03050000
 	wchar_t		*wide_name;		//!< Special wide char encoding of radiusd name.
-	wchar_t		*wide_path;		//!< Special wide char encoding of radiusd path.
 #endif
 	PyObject	*module;		//!< Local, interpreter specific module, containing
 						//!< FreeRADIUS functions.
@@ -96,6 +96,7 @@ typedef struct rlm_python_t {
 	PyObject	*pythonconf_dict;	//!< Configuration parameters defined in the module
 						//!< made available to the python script.
 	bool 		pass_all_vps;		//!< Pass all VPS lists (request, reply, config, state, proxy_req, proxy_reply)
+	bool 		pass_all_vps_dict;		//!< Pass all VPS lists as a dictionary rather than a tuple
 } rlm_python_t;
 
 /** Tracks a python module inst/thread state pair
@@ -136,6 +137,7 @@ static CONF_PARSER module_config[] = {
 	{ "python_path", FR_CONF_OFFSET(PW_TYPE_STRING, rlm_python_t, python_path), NULL },
 	{ "cext_compat", FR_CONF_OFFSET(PW_TYPE_BOOLEAN, rlm_python_t, cext_compat), "yes" },
 	{ "pass_all_vps", FR_CONF_OFFSET(PW_TYPE_BOOLEAN, rlm_python_t, pass_all_vps), "no" },
+	{ "pass_all_vps_dict", FR_CONF_OFFSET(PW_TYPE_BOOLEAN, rlm_python_t, pass_all_vps_dict), "no" },
 
 	CONF_PARSER_TERMINATOR
 };
@@ -216,23 +218,38 @@ static PyMethodDef module_methods[] = {
  */
 static void python_error_log(void)
 {
-	PyObject *pType = NULL, *pValue = NULL, *pTraceback = NULL, *pStr1 = NULL, *pStr2 = NULL;
+	PyObject *p_type = NULL, *p_value = NULL, *p_traceback = NULL, *p_str_1 = NULL, *p_str_2 = NULL;
 
-	PyErr_Fetch(&pType, &pValue, &pTraceback);
-	if (!pType || !pValue)
-		goto failed;
-	if (((pStr1 = PyObject_Str(pType)) == NULL) ||
-	    ((pStr2 = PyObject_Str(pValue)) == NULL))
-		goto failed;
+	PyErr_Fetch(&p_type, &p_value, &p_traceback);
+	PyErr_NormalizeException(&p_type, &p_value, &p_traceback);
+	if (!p_type || !p_value) goto failed;
 
-	ERROR("%s (%s)", PyString_AsString(pStr1), PyString_AsString(pStr2));
+	if (((p_str_1 = PyObject_Str(p_type)) == NULL) || ((p_str_2 = PyObject_Str(p_value)) == NULL)) goto failed;
+
+	ERROR("%s (%s)", PyString_AsString(p_str_1), PyString_AsString(p_str_2));
+
+	if (p_traceback != Py_None) {
+		PyTracebackObject *ptb = (PyTracebackObject*)p_traceback;
+		size_t fnum = 0;
+
+		for (; ptb != NULL; ptb = ptb->tb_next, fnum++) {
+			PyFrameObject *cur_frame = ptb->tb_frame;
+
+			ERROR("[%ld] %s:%d at %s()",
+				fnum,
+				PyString_AsString(cur_frame->f_code->co_filename),
+				PyFrame_GetLineNumber(cur_frame),
+				PyString_AsString(cur_frame->f_code->co_name)
+			);
+		}
+	}
 
 failed:
-	Py_XDECREF(pStr1);
-	Py_XDECREF(pStr2);
-	Py_XDECREF(pType);
-	Py_XDECREF(pValue);
-	Py_XDECREF(pTraceback);
+	Py_XDECREF(p_str_1);
+	Py_XDECREF(p_str_2);
+	Py_XDECREF(p_type);
+	Py_XDECREF(p_value);
+	Py_XDECREF(p_traceback);
 }
 
 static void mod_vptuple(TALLOC_CTX *ctx, REQUEST *request, VALUE_PAIR **vps, PyObject *pValue,
@@ -250,7 +267,7 @@ static void mod_vptuple(TALLOC_CTX *ctx, REQUEST *request, VALUE_PAIR **vps, PyO
 	 *	If the Python function gave us None for the tuple,
 	 *	then just return.
 	 */
-	if (pValue == Py_None) return;
+	if (pValue == Py_None || pValue == NULL) return;
 
 	if (!PyTuple_CheckExact(pValue)) {
 		ERROR("%s - non-tuple passed to %s", funcname, list_name);
@@ -438,10 +455,11 @@ error:
 	return false;
 }
 
-static rlm_rcode_t do_python_single(REQUEST *request, PyObject *pFunc, char const *funcname, bool pass_all_vps)
+static rlm_rcode_t do_python_single(REQUEST *request, PyObject *pFunc, char const *funcname, bool pass_all_vps, bool pass_all_vps_dict)
 {
 	PyObject	*pRet = NULL;
 	PyObject	*pArgs = NULL;
+	PyObject 	*pDictInput = NULL;
 	int		ret;
 	int 		i;
 
@@ -469,15 +487,21 @@ static rlm_rcode_t do_python_single(REQUEST *request, PyObject *pFunc, char cons
 
 		/* fill proxy vps */
 		if (request->proxy) {
-			if (!mod_populate_vps(pArgs, 4, request->proxy->vps) ||
-			    !mod_populate_vps(pArgs, 5, request->proxy_reply->vps)) {
+			if (!mod_populate_vps(pArgs, 4, request->proxy->vps)) {
 				ret = RLM_MODULE_FAIL;
 				goto finish;
 			}
-		}
-		/* If there are no proxy lists */
-		else {
+		} else {
 			mod_populate_vps(pArgs, 4, NULL);
+		}
+
+		/* fill proxy_reply vps */
+		if (request->proxy_reply) {
+			if (!mod_populate_vps(pArgs, 5, request->proxy_reply->vps)) {
+				ret = RLM_MODULE_FAIL;
+				goto finish;
+			}
+		} else {
 			mod_populate_vps(pArgs, 5, NULL);
 		}
 
@@ -486,12 +510,28 @@ static rlm_rcode_t do_python_single(REQUEST *request, PyObject *pFunc, char cons
 	else for (i = 0; i < 6; i++) mod_populate_vps(pArgs, i, NULL);
 
 	/*
-	 * Call Python function. If pass_all_vps is true, a 6-tuple representing
-	 * (Request, Reply, Config, State, Proxy-Request, Proxy-Reply) is passed
-	 * as argument to the module callback.
-	 * Otherwise, a tuple representing just the request is passed.
+	 * Call Python function. If pass_all_vps_dict is true, a dictionary with the
+	 * appropriate "request", "reply"... keys is passed as argument to the
+	 * module callback.
+	 * Else, if pass_all_vps is true, a 6-tuple representing
+	 * (Request, Reply, Config, State, Proxy-Request, Proxy-Reply) is passed.
+	 * Otherwise, a tuple representing just the request is used.
 	 */
-	if (pass_all_vps)
+	if (pass_all_vps_dict) {
+		pDictInput = PyDict_New();
+		if (pDictInput == NULL ||
+		    PyDict_SetItemString(pDictInput, "request", PyTuple_GET_ITEM(pArgs, 0)) ||
+		    PyDict_SetItemString(pDictInput, "reply", PyTuple_GET_ITEM(pArgs, 1)) ||
+		    PyDict_SetItemString(pDictInput, "config", PyTuple_GET_ITEM(pArgs, 2)) ||
+		    PyDict_SetItemString(pDictInput, "session-state", PyTuple_GET_ITEM(pArgs, 3)) ||
+		    PyDict_SetItemString(pDictInput, "proxy-request", PyTuple_GET_ITEM(pArgs, 4)) ||
+		    PyDict_SetItemString(pDictInput, "proxy-reply", PyTuple_GET_ITEM(pArgs, 5))) {
+			ret = RLM_MODULE_FAIL;
+			goto finish;
+		}
+		pRet = PyObject_CallFunctionObjArgs(pFunc, pDictInput, NULL);
+	}
+	else if (pass_all_vps)
 		pRet = PyObject_CallFunctionObjArgs(pFunc, pArgs, NULL);
 	else
 		pRet = PyObject_CallFunctionObjArgs(pFunc, PyTuple_GET_ITEM(pArgs, 0), NULL);
@@ -522,9 +562,10 @@ static rlm_rcode_t do_python_single(REQUEST *request, PyObject *pFunc, char cons
 	 */
 	if (PyTuple_CheckExact(pRet)) {
 		PyObject *pTupleInt;
+		int tuple_size = PyTuple_GET_SIZE(pRet);
 
-		if (PyTuple_GET_SIZE(pRet) != 3) {
-			ERROR("%s - Tuple must be (return, replyTuple, configTuple)", funcname);
+		if (tuple_size < 2 || tuple_size > 3) {
+			ERROR("%s - Tuple must be (return, updateDict) or (return, replyTuple, configTuple)", funcname);
 			ret = RLM_MODULE_FAIL;
 			goto finish;
 		}
@@ -537,13 +578,49 @@ static rlm_rcode_t do_python_single(REQUEST *request, PyObject *pFunc, char cons
 		}
 		/* Now have the return value */
 		ret = PyInt_AsLong(pTupleInt);
-		/* Reply item tuple */
-		mod_vptuple(request->reply, request, &request->reply->vps,
-			    PyTuple_GET_ITEM(pRet, 1), funcname, "reply");
-		/* Config item tuple */
-		mod_vptuple(request, request, &request->config,
-			    PyTuple_GET_ITEM(pRet, 2), funcname, "config");
 
+		/* process updateDict */
+		if (tuple_size == 2) {
+			PyObject *updateDict = PyTuple_GET_ITEM(pRet, 1);
+			if (!PyDict_CheckExact(updateDict)) {
+				ERROR("%s - updateDict is not a dictionary", funcname);
+				ret = RLM_MODULE_FAIL;
+				goto finish;
+			}
+			mod_vptuple(request->reply, request, &request->reply->vps,
+				    PyDict_GetItemString(updateDict, "reply"), funcname, "reply");
+			mod_vptuple(request, request, &request->config,
+				    PyDict_GetItemString(updateDict, "config"), funcname, "config");
+			mod_vptuple(request->packet, request, &request->packet->vps,
+				    PyDict_GetItemString(updateDict, "request"), funcname, "request");
+			mod_vptuple(request->state_ctx, request, &request->state,
+				    PyDict_GetItemString(updateDict, "session-state"), funcname, "session-state");
+#ifdef WITH_PROXY
+			if (request->proxy)
+				mod_vptuple(request->proxy, request, &request->proxy->vps,
+					    PyDict_GetItemString(updateDict, "proxy-request"), funcname, "proxy-request");
+			if (request->proxy_reply)
+				mod_vptuple(request->proxy_reply, request, &request->proxy_reply->vps,
+					    PyDict_GetItemString(updateDict, "proxy-reply"), funcname, "proxy-reply");
+#endif
+			/*
+			 *	Update cached copies
+			 */
+			request->username = fr_pair_find_by_num(request->packet->vps, PW_USER_NAME, 0, TAG_ANY);
+			request->password = fr_pair_find_by_num(request->packet->vps, PW_USER_PASSWORD, 0, TAG_ANY);
+			if (!request->password)
+				request->password = fr_pair_find_by_num(request->packet->vps, PW_CHAP_PASSWORD, 0, TAG_ANY);
+		}
+
+		/* process replyTuple and configTuple */
+		else if (tuple_size == 3) {
+			/* Reply item tuple */
+			mod_vptuple(request->reply, request, &request->reply->vps,
+				    PyTuple_GET_ITEM(pRet, 1), funcname, "reply");
+			/* Config item tuple */
+			mod_vptuple(request, request, &request->config,
+				    PyTuple_GET_ITEM(pRet, 2), funcname, "config");
+		}
 	} else if (PyInt_CheckExact(pRet)) {
 		/* Just an integer */
 		ret = PyInt_AsLong(pRet);
@@ -562,6 +639,7 @@ static rlm_rcode_t do_python_single(REQUEST *request, PyObject *pFunc, char cons
 finish:
 	Py_XDECREF(pArgs);
 	Py_XDECREF(pRet);
+	Py_XDECREF(pDictInput);
 
 	return ret;
 }
@@ -691,7 +769,8 @@ static rlm_rcode_t do_python(rlm_python_t *inst, REQUEST *request, PyObject *pFu
 	RDEBUG3("Using thread state %p", this_thread->state);
 
 	PyEval_RestoreThread(this_thread->state);	/* Swap in our local thread state */
-	ret = do_python_single(request, pFunc, funcname, inst->pass_all_vps);
+	ret = do_python_single(request, pFunc, funcname, inst->pass_all_vps, inst->pass_all_vps_dict);
+	if (ret == RLM_MODULE_FAIL) python_error_log();
 	PyEval_SaveThread();
 
 	return ret;
@@ -969,19 +1048,19 @@ static int python_interpreter_init(rlm_python_t *inst, CONF_SECTION *conf)
 		 *	the lifetime of the module.
 		 */
 		if (inst->python_path) {
-#if PY_VERSION_HEX > 0x03050000
-			{
-				inst->wide_path = Py_DecodeLocale(inst->python_path, strlen(inst->python_path));
-				PySys_SetPath(inst->wide_path);
-			}
-#else
-			{
-				char *path;
+			char *p, *path;
+			PyObject *sys = PyImport_ImportModule("sys");
+			PyObject *sys_path = PyObject_GetAttrString(sys, "path");
 
-				memcpy(&path, &inst->python_path, sizeof(path));
-				PySys_SetPath(path);
+			memcpy(&p, &inst->python_path, sizeof(path));
+
+			for (path = strtok(p, ":"); path != NULL; path = strtok(NULL, ":")) {
+				PyList_Append(sys_path, PyString_FromString(path));
 			}
-#endif
+
+			PyObject_SetAttrString(sys, "path", sys_path);
+			Py_DecRef(sys);
+			Py_DecRef(sys_path);
 		}
 
 		/*
@@ -1091,8 +1170,8 @@ static int mod_instantiate(CONF_SECTION *conf, void *instance)
 	/*
 	 *	Call the instantiate function.
 	 */
-	code = do_python_single(NULL, inst->instantiate.function, "instantiate", inst->pass_all_vps);
-	if (code < 0) {
+	code = do_python_single(NULL, inst->instantiate.function, "instantiate", inst->pass_all_vps, inst->pass_all_vps_dict);
+	if (code == RLM_MODULE_FAIL) {
 	error:
 		python_error_log();	/* Needs valid thread with GIL */
 		PyEval_SaveThread();
@@ -1113,7 +1192,8 @@ static int mod_detach(void *instance)
 	 */
 	PyEval_RestoreThread(inst->sub_interpreter);
 
-	ret = do_python_single(NULL, inst->detach.function, "detach", inst->pass_all_vps);
+	ret = do_python_single(NULL, inst->detach.function, "detach", inst->pass_all_vps, inst->pass_all_vps_dict);
+	if (ret == RLM_MODULE_FAIL) python_error_log();
 
 #define PYTHON_FUNC_DESTROY(_x) python_function_destroy(&inst->_x)
 	PYTHON_FUNC_DESTROY(instantiate);
@@ -1150,7 +1230,6 @@ static int mod_detach(void *instance)
 
 #if PY_VERSION_HEX > 0x03050000
 		if (inst->wide_name) PyMem_RawFree(inst->wide_name);
-		if (inst->wide_path) PyMem_RawFree(inst->wide_path);
 #endif
 	}
 
@@ -1171,7 +1250,7 @@ extern module_t rlm_python;
 module_t rlm_python = {
 	.magic		= RLM_MODULE_INIT,
 	.name		= "python",
-	.type		= RLM_TYPE_THREAD_SAFE,
+	.type		= RLM_TYPE_THREAD_UNSAFE,
 	.inst_size	= sizeof(rlm_python_t),
 	.config		= module_config,
 	.instantiate	= mod_instantiate,
