@@ -15,7 +15,7 @@
  */
 
 /**
- * $Id: 1b8f826ad1f8eeb4505087172ee7d22bbfa52c4b $
+ * $Id: 0c872ea24653a1b68a71a56072b4324fe06e321a $
  * @file rlm_sql.c
  * @brief Implements SQL 'users' file, and SQL accounting.
  *
@@ -24,7 +24,7 @@
  * @copyright 2000  Mike Machado <mike@innercite.com>
  * @copyright 2000  Alan DeKok <aland@ox.org>
  */
-RCSID("$Id: 1b8f826ad1f8eeb4505087172ee7d22bbfa52c4b $")
+RCSID("$Id: 0c872ea24653a1b68a71a56072b4324fe06e321a $")
 
 #include <ctype.h>
 
@@ -110,6 +110,7 @@ static const CONF_PARSER module_config[] = {
 #endif
 	{ "safe-characters", FR_CONF_OFFSET(PW_TYPE_STRING | PW_TYPE_DEPRECATED, rlm_sql_config_t, allowed_chars), NULL },
 	{ "safe_characters", FR_CONF_OFFSET(PW_TYPE_STRING, rlm_sql_config_t, allowed_chars), "@abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.-_: /" },
+	{ "auto_escape", FR_CONF_OFFSET(PW_TYPE_BOOLEAN, rlm_sql_config_t, driver_specific_escape), "no" },
 
 	/*
 	 *	This only works for a few drivers.
@@ -122,6 +123,8 @@ static const CONF_PARSER module_config[] = {
 	CONF_PARSER_TERMINATOR
 };
 
+static size_t sql_escape_for_xlat_func(REQUEST *request, char *out, size_t outlen, char const *in, void *arg);
+
 /*
  *	Fall-Through checking function from rlm_files.c
  */
@@ -375,7 +378,8 @@ static int generate_sql_clients(rlm_sql_t *inst)
 static size_t sql_escape_func(UNUSED REQUEST *request, char *out, size_t outlen,
 			      char const *in, void *arg)
 {
-	rlm_sql_t *inst = arg;
+	rlm_sql_handle_t *handle = talloc_get_type_abort(arg, rlm_sql_handle_t);
+	rlm_sql_t *inst = handle->inst;
 	size_t len = 0;
 
 	while (in[0]) {
@@ -478,6 +482,28 @@ static size_t sql_escape_func(UNUSED REQUEST *request, char *out, size_t outlen,
 	return len;
 }
 
+/** Passed as the escape function to map_proc and sql xlat methods
+ *
+ * The variant reserves a connection for the escape functions to use, and releases it after
+ * escaping is complete.
+ */
+static size_t sql_escape_for_xlat_func(REQUEST *request, char *out, size_t outlen, char const *in, void *arg)
+{
+	size_t			ret;
+	rlm_sql_t		*inst = talloc_get_type_abort(arg, rlm_sql_t);
+	rlm_sql_handle_t	*handle;
+
+	handle = fr_connection_get(inst->pool);
+	if (!handle) {
+		out[0] = '\0';
+		return 0;
+	}
+	ret = inst->sql_escape_func(request, out, outlen, in, handle);
+	fr_connection_release(inst->pool, handle);
+
+	return ret;
+}
+
 /*
  *	Set the SQL user name.
  *
@@ -547,7 +573,7 @@ static int sql_get_grouplist(rlm_sql_t *inst, rlm_sql_handle_t **handle, REQUEST
 
 	if (!inst->config->groupmemb_query) return 0;
 
-	if (radius_axlat(&expanded, request, inst->config->groupmemb_query, sql_escape_func, inst) < 0) return -1;
+	if (radius_axlat(&expanded, request, inst->config->groupmemb_query, sql_escape_for_xlat_func, inst) < 0) return -1;
 
 	ret = rlm_sql_select_query(inst, request, handle, expanded);
 	talloc_free(expanded);
@@ -726,7 +752,7 @@ static rlm_rcode_t rlm_sql_process_groups(rlm_sql_t *inst, REQUEST *request, rlm
 			 *	Expand the group query
 			 */
 			if (radius_axlat(&expanded, request, inst->config->authorize_group_check_query,
-					 sql_escape_func, inst) < 0) {
+					 inst->sql_escape_func, *handle) < 0) {
 				REDEBUG("Error generating query");
 				rcode = RLM_MODULE_FAIL;
 				goto finish;
@@ -776,7 +802,7 @@ static rlm_rcode_t rlm_sql_process_groups(rlm_sql_t *inst, REQUEST *request, rlm
 			 *	Now get the reply pairs since the paircompare matched
 			 */
 			if (radius_axlat(&expanded, request, inst->config->authorize_group_reply_query,
-					 sql_escape_func, inst) < 0) {
+					 inst->sql_escape_func, *handle) < 0) {
 				REDEBUG("Error generating query");
 				rcode = RLM_MODULE_FAIL;
 				goto finish;
@@ -858,8 +884,6 @@ static int mod_bootstrap(CONF_SECTION *conf, void *instance)
 
 	/*
 	 *	Load the appropriate driver for our database.
-	 *
-	 *	We need this to check if the sql_fields callback is provided.
 	 */
 	inst->handle = fr_dlopenext(inst->config->sql_driver_name);
 	if (!inst->handle) {
@@ -913,7 +937,7 @@ static int mod_bootstrap(CONF_SECTION *conf, void *instance)
 	/*
 	 *	Register the SQL xlat function
 	 */
-	xlat_register(inst->name, sql_xlat, sql_escape_func, inst);
+	xlat_register(inst->name, sql_xlat, sql_escape_for_xlat_func, inst);
 
 	return 0;
 }
@@ -1043,11 +1067,18 @@ do { \
 	 *	Export these methods, too.  This avoids RTDL_GLOBAL.
 	 */
 	inst->sql_set_user		= sql_set_user;
-	inst->sql_escape_func		= sql_escape_func;
 	inst->sql_query			= rlm_sql_query;
 	inst->sql_select_query		= rlm_sql_select_query;
 	inst->sql_fetch_row		= rlm_sql_fetch_row;
 
+	/*
+	 *	Either use the module specific escape function
+	 *	or our default one.
+	 */
+	inst->sql_escape_func = inst->module->sql_escape_func && inst->config->driver_specific_escape ?
+				inst->module->sql_escape_func :
+				sql_escape_func;
+
 	if (inst->module->mod_instantiate) {
 		CONF_SECTION *cs;
 		char const *name;
@@ -1156,7 +1187,7 @@ static rlm_rcode_t mod_authorize(void *instance, REQUEST *request)
 		VALUE_PAIR *vp;
 
 		if (radius_axlat(&expanded, request, inst->config->authorize_check_query,
-				 sql_escape_func, inst) < 0) {
+				 inst->sql_escape_func, handle) < 0) {
 			REDEBUG("Error generating query");
 			rcode = RLM_MODULE_FAIL;
 			goto error;
@@ -1170,7 +1201,10 @@ static rlm_rcode_t mod_authorize(void *instance, REQUEST *request)
 			goto error;
 		}
 
-		if (rows == 0) goto skipreply;	/* Don't need to free VPs we don't have */
+		if (rows == 0) {
+			RWDEBUG2("User not found in radcheck table.");
+			goto skipreply;	/* Don't need to free VPs we don't have */
+		}
 
 		/*
 		 *	Only do this if *some* check pairs were returned
@@ -1178,6 +1212,7 @@ static rlm_rcode_t mod_authorize(void *instance, REQUEST *request)
 		RDEBUG2("User found in radcheck table");
 		user_found = true;
 		if (paircompare(request, request->packet->vps, check_tmp, &request->reply->vps) != 0) {
+			RWDEBUG2("check items do not match.");
 			fr_pair_list_free(&check_tmp);
 			check_tmp = NULL;
 			goto skipreply;
@@ -1204,7 +1239,7 @@ static rlm_rcode_t mod_authorize(void *instance, REQUEST *request)
 		 *	Now get the reply pairs since the paircompare matched
 		 */
 		if (radius_axlat(&expanded, request, inst->config->authorize_reply_query,
-				 sql_escape_func, inst) < 0) {
+				 inst->sql_escape_func, handle) < 0) {
 			REDEBUG("Error generating query");
 			rcode = RLM_MODULE_FAIL;
 			goto error;
@@ -1233,12 +1268,6 @@ static rlm_rcode_t mod_authorize(void *instance, REQUEST *request)
 		reply_tmp = NULL;
 	}
 
-	/*
-	 *	Neither group checks nor profiles will work without
-	 *	a group membership query.
-	 */
-	if (!inst->config->groupmemb_query) goto release;
-
 skipreply:
 	if ((do_fall_through == FALL_THROUGH_YES) ||
 	    (inst->config->read_groups && (do_fall_through == FALL_THROUGH_DEFAULT))) {
@@ -1430,7 +1459,7 @@ static int acct_redundant(rlm_sql_t *inst, REQUEST *request, sql_acct_section_t
 			goto finish;
 		}
 
-		if (radius_axlat(&expanded, request, value, sql_escape_func, inst) < 0) {
+		if (radius_axlat(&expanded, request, value, inst->sql_escape_func, handle) < 0) {
 			rcode = RLM_MODULE_FAIL;
 
 			goto finish;
@@ -1582,11 +1611,6 @@ static rlm_rcode_t mod_checksimul(void *instance, REQUEST * request)
 		return RLM_MODULE_FAIL;
 	}
 
-	if (radius_axlat(&expanded, request, inst->config->simul_count_query, sql_escape_func, inst) < 0) {
-		sql_unset_user(inst, request);
-		return RLM_MODULE_FAIL;
-	}
-
 	/* initialize the sql socket */
 	handle = fr_connection_get(inst->pool);
 	if (!handle) {
@@ -1595,6 +1619,11 @@ static rlm_rcode_t mod_checksimul(void *instance, REQUEST * request)
 		return RLM_MODULE_FAIL;
 	}
 
+	if (radius_axlat(&expanded, request, inst->config->simul_count_query, inst->sql_escape_func, handle) < 0) {
+		sql_unset_user(inst, request);
+		return RLM_MODULE_FAIL;
+	}
+
 	if (rlm_sql_select_query(inst, request, &handle, expanded) != RLM_SQL_OK) {
 		rcode = RLM_MODULE_FAIL;
 		goto release;	/* handle may no longer be valid */
@@ -1632,7 +1661,7 @@ static rlm_rcode_t mod_checksimul(void *instance, REQUEST * request)
 		goto finish;
 	}
 
-	if (radius_axlat(&expanded, request, inst->config->simul_verify_query, sql_escape_func, inst) < 0) {
+	if (radius_axlat(&expanded, request, inst->config->simul_verify_query, inst->sql_escape_func, handle) < 0) {
 		rcode = RLM_MODULE_FAIL;
 
 		goto finish;
